/* ***********************************************************
 * This file was automatically generated on 2012-05-18.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for controlling Stacks and four Bricklets
	/// </summary>
	public class BrickMaster : Device 
	{
		private static byte FUNCTION_GET_STACK_VOLTAGE = 1;
		private static byte FUNCTION_GET_STACK_CURRENT = 2;
		private static byte FUNCTION_SET_EXTENSION_TYPE = 3;
		private static byte FUNCTION_GET_EXTENSION_TYPE = 4;
		private static byte FUNCTION_IS_CHIBI_PRESENT = 5;
		private static byte FUNCTION_SET_CHIBI_ADDRESS = 6;
		private static byte FUNCTION_GET_CHIBI_ADDRESS = 7;
		private static byte FUNCTION_SET_CHIBI_MASTER_ADDRESS = 8;
		private static byte FUNCTION_GET_CHIBI_MASTER_ADDRESS = 9;
		private static byte FUNCTION_SET_CHIBI_SLAVE_ADDRESS = 10;
		private static byte FUNCTION_GET_CHIBI_SLAVE_ADDRESS = 11;
		private static byte FUNCTION_GET_CHIBI_SIGNAL_STRENGTH = 12;
		private static byte FUNCTION_GET_CHIBI_ERROR_LOG = 13;
		private static byte FUNCTION_SET_CHIBI_FREQUENCY = 14;
		private static byte FUNCTION_GET_CHIBI_FREQUENCY = 15;
		private static byte FUNCTION_SET_CHIBI_CHANNEL = 16;
		private static byte FUNCTION_GET_CHIBI_CHANNEL = 17;


		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickMaster(string uid) : base(uid) 
		{
			this.expectedName = "Master Brick";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 1;
			this.bindingVersion[2] = 0;

		}

		/// <summary>
		///  Returns the stack voltage in mV. The stack voltage is the
		///  voltage that is supplied via the stack, i.e. it is given by a 
		///  Step-Down or Step-Up Power Supply.
		/// </summary>
		public ushort GetStackVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_STACK_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_STACK_VOLTAGE, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Returns the stack current in mA. The stack current is the
		///  current that is drawn via the stack, i.e. it is given by a
		///  Step-Down or Step-Up Power Supply.
		/// </summary>
		public ushort GetStackCurrent()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_STACK_CURRENT, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_STACK_CURRENT, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Writes the extension type to the EEPROM of a specified extension. 
		///  The extension is either 0 or 1 (0 is the on the bottom, 1 is the on on top, 
		///  if only one extension is present use 0).
		///  
		///  Possible extension types:
		///  
		///  <code>
		///   :header: "Type", "Description"
		///   :widths: 10, 100
		///   "1", "Chibi"
		///   "2", "RS485"
		///  </code>
		///  
		///  The extension type is already set when bought and it can be set with the 
		///  Brick Viewer, it is unlikely that you need this function.
		///  
		///  The value will be saved in the EEPROM of the Chibi Extension, it does not
		///  have to be set on every startup.
		/// </summary>
		public void SetExtensionType(byte extension, uint exttype)
		{
			byte[] data_ = new byte[9];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_EXTENSION_TYPE, 1, data_);
			LEConverter.To((ushort)9, 2, data_);
			LEConverter.To(extension, 4, data_);
			LEConverter.To(exttype, 5, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the extension type for a given extension as set by 
		///  <see cref="Tinkerforge.BrickMaster.SetExtensionType"/>.
		/// </summary>
		public uint GetExtensionType(byte extension)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_EXTENSION_TYPE, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(extension, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_EXTENSION_TYPE, out answer);

			return LEConverter.UIntFrom(4, answer);
		}

		/// <summary>
		///  Returns true if a Chibi Extension is available to be used by the Master.
		/// </summary>
		public bool IsChibiPresent()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_IS_CHIBI_PRESENT, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_IS_CHIBI_PRESENT, out answer);

			return LEConverter.BoolFrom(4, answer);
		}

		/// <summary>
		///  Sets the address (1-255) belonging to the Chibi Extension.
		///  
		///  It is possible to set the address with the Brick Viewer and it will be 
		///  saved in the EEPROM of the Chibi Extension, it does not
		///  have to be set on every startup.
		/// </summary>
		public void SetChibiAddress(byte address)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_CHIBI_ADDRESS, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(address, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the address as set by <see cref="Tinkerforge.BrickMaster.SetChibiAddress"/>.
		/// </summary>
		public byte GetChibiAddress()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CHIBI_ADDRESS, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_CHIBI_ADDRESS, out answer);

			return LEConverter.ByteFrom(4, answer);
		}

		/// <summary>
		///  Sets the address (1-255) of the Chibi Master. This address is used if the
		///  Chibi Extension is used as slave (i.e. it does not have a USB connection).
		///  
		///  It is possible to set the address with the Brick Viewer and it will be 
		///  saved in the EEPROM of the Chibi Extension, it does not
		///  have to be set on every startup.
		/// </summary>
		public void SetChibiMasterAddress(byte address)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_CHIBI_MASTER_ADDRESS, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(address, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the address as set by <see cref="Tinkerforge.BrickMaster.SetChibiMasterAddress"/>.
		/// </summary>
		public byte GetChibiMasterAddress()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CHIBI_MASTER_ADDRESS, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_CHIBI_MASTER_ADDRESS, out answer);

			return LEConverter.ByteFrom(4, answer);
		}

		/// <summary>
		///  Sets up to 256 slave addresses. The address numeration has to be used 
		///  ascending from 0. For example: If you use the Chibi Extension in Master mode
		///  (i.e. the stack has an USB connection) and you want to talk to three other
		///  Chibi stacks with the IDs 17, 23, and 42, you should call with "(0, 17),
		///  (1, 23) and (2, 42)".
		///  
		///  It is possible to set the addresses with the Brick Viewer and it will be 
		///  saved in the EEPROM of the Chibi Extension, they don't
		///  have to be set on every startup.
		/// </summary>
		public void SetChibiSlaveAddress(byte num, byte address)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_CHIBI_SLAVE_ADDRESS, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(num, 4, data_);
			LEConverter.To(address, 5, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the slave address for a given num as set by 
		///  <see cref="Tinkerforge.BrickMaster.SetChibiSlaveAddress"/>.
		/// </summary>
		public byte GetChibiSlaveAddress(byte num)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CHIBI_SLAVE_ADDRESS, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(num, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_CHIBI_SLAVE_ADDRESS, out answer);

			return LEConverter.ByteFrom(4, answer);
		}

		/// <summary>
		///  Returns the signal strength in dBm. The signal strength updates every time a
		///  packet is received.
		/// </summary>
		public byte GetChibiSignalStrength()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CHIBI_SIGNAL_STRENGTH, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_CHIBI_SIGNAL_STRENGTH, out answer);

			return LEConverter.ByteFrom(4, answer);
		}

		/// <summary>
		///  Returns underrun, CRC error, no ACK and overflow error counts of the Chibi
		///  communication. If these errors start rising, it is likely that either the
		///  distance between two Chibi stacks is becoming too big or there are
		///  interferences.
		/// </summary>
		public void GetChibiErrorLog(out ushort underrun, out ushort crcError, out ushort noAck, out ushort overflow)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CHIBI_ERROR_LOG, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_CHIBI_ERROR_LOG, out answer);

			underrun = LEConverter.UShortFrom(4, answer);
			crcError = LEConverter.UShortFrom(6, answer);
			noAck = LEConverter.UShortFrom(8, answer);
			overflow = LEConverter.UShortFrom(10, answer);
		}

		/// <summary>
		///  Sets the Chibi frequency range for the Chibi Extension. Possible values are:
		///  
		///  <code>
		///   :header: "Type", "Description"
		///   :widths: 10, 100
		///   "0", "OQPSK 868Mhz (Europe)"
		///   "1", "OQPSK 915Mhz (US)"
		///   "2", "OQPSK 780Mhz (China)"
		///   "3", "BPSK40 915Mhz"
		///  </code>
		///  
		///  It is possible to set the frequency with the Brick Viewer and it will be 
		///  saved in the EEPROM of the Chibi Extension, it does not
		///  have to be set on every startup.
		/// </summary>
		public void SetChibiFrequency(byte frequency)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_CHIBI_FREQUENCY, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(frequency, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the frequency value as set by <see cref="Tinkerforge.BrickMaster.SetChibiFrequency"/>.
		/// </summary>
		public byte GetChibiFrequency()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CHIBI_FREQUENCY, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_CHIBI_FREQUENCY, out answer);

			return LEConverter.ByteFrom(4, answer);
		}

		/// <summary>
		///  Sets the channel used by the Chibi Extension. Possible channels are
		///  different for different frequencies:
		///  
		///  <code>
		///   :header: "Frequency", "Possible Channels"
		///   :widths: 40, 60
		///   "OQPSK 868Mhz (Europe)", "0"
		///   "OQPSK 915Mhz (US)", "1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
		///   "OQPSK 780Mhz (China)", "0, 1, 2, 3"
		///   "BPSK40 915Mhz", "1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
		///  </code>
		///  
		///  It is possible to set the frequency with the Brick Viewer and it will be 
		///  saved in the EEPROM of the Chibi Extension, it does not
		///  have to be set on every startup.
		/// </summary>
		public void SetChibiChannel(byte channel)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_CHIBI_CHANNEL, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(channel, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the channel as set by <see cref="Tinkerforge.BrickMaster.SetChibiChannel"/>.
		/// </summary>
		public byte GetChibiChannel()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CHIBI_CHANNEL, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_CHIBI_CHANNEL, out answer);

			return LEConverter.ByteFrom(4, answer);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetStackVoltage(out ushort voltage)
		{
			voltage = GetStackVoltage();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetStackCurrent(out ushort current)
		{
			current = GetStackCurrent();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetExtensionType(byte extension, out uint exttype)
		{
			exttype = GetExtensionType(extension);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void IsChibiPresent(out bool present)
		{
			present = IsChibiPresent();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetChibiAddress(out byte address)
		{
			address = GetChibiAddress();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetChibiMasterAddress(out byte address)
		{
			address = GetChibiMasterAddress();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetChibiSlaveAddress(byte num, out byte address)
		{
			address = GetChibiSlaveAddress(num);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetChibiSignalStrength(out byte signalStrength)
		{
			signalStrength = GetChibiSignalStrength();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetChibiFrequency(out byte frequency)
		{
			frequency = GetChibiFrequency();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetChibiChannel(out byte channel)
		{
			channel = GetChibiChannel();
		}
	}
}
