/* ***********************************************************
 * This file was automatically generated on 2012-05-18.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for controlling up to 16 general purpose input/output pins
	/// </summary>
	public class BrickletIO16 : Device 
	{
		private static byte FUNCTION_SET_PORT = 1;
		private static byte FUNCTION_GET_PORT = 2;
		private static byte FUNCTION_SET_PORT_CONFIGURATION = 3;
		private static byte FUNCTION_GET_PORT_CONFIGURATION = 4;
		private static byte FUNCTION_SET_DEBOUNCE_PERIOD = 5;
		private static byte FUNCTION_GET_DEBOUNCE_PERIOD = 6;
		private static byte FUNCTION_SET_PORT_INTERRUPT = 7;
		private static byte FUNCTION_GET_PORT_INTERRUPT = 8;
		private static byte CALLBACK_INTERRUPT = 9;


		/// <summary>
		///  This callback is triggered whenever a change of the voltage level is detected
		///  on pins where the interrupt was activated with <see cref="Tinkerforge.BrickletIO16.SetPortInterrupt"/>.
		///  
		///  The values are the port, a bit mask that specifies which interrupts occurred
		///  and the current value bit mask of the port.
		///  
		///  For example:
		///  
		///   * ("a", 1, 1) means that on port a an interrupt on pin 0 occurred and
		///     currently pin 0 is high and pins 1-7 are low.
		///   * ("b", 128, 254) means that on port b an interrupt on pins 0 and 7
		///     occurred and currently pin 0 is low and pins 1-7 are high.
		/// </summary>
		public delegate void Interrupt(char port, byte interruptMask, byte valueMask);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickletIO16(string uid) : base(uid) 
		{
			this.expectedName = "IO-16 Bricklet";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 0;
			messageCallbacks[CALLBACK_INTERRUPT] = new MessageCallback(CallbackInterrupt);
		}

		/// <summary>
		///  Sets the output value (high or low) for a port ("a" or "b") with a bit mask. 
		///  The bit mask is 8 bit long, "true" refers to high and "false" refers to low.
		///  
		///  For example: The bitstring "11110000" will turn the pins 0-3 high and the
		///  pins 4-7 low for the specified port.
		///  
		///  <note>
		///    This function does nothing for pins that are configured as input.
		///    Pull up resistors can be switched on with <see cref="Tinkerforge.BrickletIO16.SetPortConfiguration"/>.
		///  </note>
		/// </summary>
		public void SetPort(char port, byte valueMask)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_PORT, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(port, 4, data_);
			LEConverter.To(valueMask, 5, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns a bit mask of the values that are currently measured on the
		///  specified port. This function works if the pin is configured to input
		///  as well as if it is configured to output.
		/// </summary>
		public byte GetPort(char port)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_PORT, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(port, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_PORT, out answer);

			return LEConverter.ByteFrom(4, answer);
		}

		/// <summary>
		///  Configures the value and direction of a specified port. Possible directions
		///  are "i" and "o" for input and output.
		///  
		///  If the direction is configured as output, the value is either high or low
		///  (set as true or false).
		///  
		///  If the direction is configured as output, the value is either pull up or
		///  default (set as true or false).
		///  
		///  For example: 
		///  
		///   * ("a", 0xFF, 'i', true) will set all pins of port a as input pull up. 
		///   * ("a", 128, 'i', false) will set pin 7 of port a as input default (floating if nothing is connected). 
		///   * ("b", 3, 'o', false) will set pins 0 and 1 of port b as output low.
		///   * ("b", 4, 'o', true) will set pin 2 of port b as output high.
		/// </summary>
		public void SetPortConfiguration(char port, byte portMask, char direction, bool value)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_PORT_CONFIGURATION, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(port, 4, data_);
			LEConverter.To(portMask, 5, data_);
			LEConverter.To(direction, 6, data_);
			LEConverter.To(value, 7, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns a value bit mask and a direction bit mask for the specified port.
		///  
		///  For example: A return value of the bitstrings "11110000" and "11001100" for
		///  direction and value means that:
		///  
		///   * pins 0 and 1 are configured as input pull up, 
		///   * pins 2 and 3 are configured as input default,
		///   * pins 4 and 5 are configured as output high
		///   * and pins 6 and 7 are configured as output low.
		/// </summary>
		public void GetPortConfiguration(char port, out byte directionMask, out byte valueMask)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_PORT_CONFIGURATION, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(port, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_PORT_CONFIGURATION, out answer);

			directionMask = LEConverter.ByteFrom(4, answer);
			valueMask = LEConverter.ByteFrom(5, answer);
		}

		/// <summary>
		///  Sets the debounce period of the <see cref="Tinkerforge.BrickletIO16.Interrupt"/> callback in ms.
		///  
		///  For example: If you set this value to 100, you will get the interrupt
		///  maximal every 100ms. This is necessary if something that bounces is
		///  connected to the IO-16 Bricklet, such as a button.
		///  
		///  The default value is 100.
		/// </summary>
		public void SetDebouncePeriod(uint debounce)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_DEBOUNCE_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(debounce, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the debounce period as set by <see cref="Tinkerforge.BrickletIO16.SetDebouncePeriod"/>.
		/// </summary>
		public uint GetDebouncePeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_DEBOUNCE_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_DEBOUNCE_PERIOD, out answer);

			return LEConverter.UIntFrom(4, answer);
		}

		/// <summary>
		///  Sets the pins on which an interrupt is activated with a bit mask. 
		///  Interrupts are triggered on changes of the voltage level of the pin,
		///  i.e. changes from high to low and low to high.
		///  
		///  For example: ('a', 129) will enable the interrupt for pins 0 and 7 of
		///  port a.
		///  
		///  The interrupt is delivered with the callback <see cref="Tinkerforge.BrickletIO16.Interrupt"/>.
		/// </summary>
		public void SetPortInterrupt(char port, byte interruptMask)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_PORT_INTERRUPT, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(port, 4, data_);
			LEConverter.To(interruptMask, 5, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the interrupt bit mask for the specified port as set by
		///  <see cref="Tinkerforge.BrickletIO16.SetPortInterrupt"/>.
		/// </summary>
		public byte GetPortInterrupt(char port)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_PORT_INTERRUPT, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(port, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_PORT_INTERRUPT, out answer);

			return LEConverter.ByteFrom(4, answer);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetPort(char port, out byte valueMask)
		{
			valueMask = GetPort(port);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetDebouncePeriod(out uint debounce)
		{
			debounce = GetDebouncePeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetPortInterrupt(char port, out byte interruptMask)
		{
			interruptMask = GetPortInterrupt(port);
		}

		internal int CallbackInterrupt(byte[] data_)
		{
			char port = LEConverter.CharFrom(4, data_);
			byte interruptMask = LEConverter.ByteFrom(5, data_);
			byte valueMask = LEConverter.ByteFrom(6, data_);

			((Interrupt)callbacks[CALLBACK_INTERRUPT])(port, interruptMask, valueMask);
			return 7;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(Interrupt))
			{
				callbacks[CALLBACK_INTERRUPT] = d;
			}
		}
	}
}
