/* ***********************************************************
 * This file was automatically generated on 2012-05-18.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for sensing Temperature
	/// </summary>
	public class BrickletTemperature : Device 
	{
		private static byte FUNCTION_GET_TEMPERATURE = 1;
		private static byte FUNCTION_SET_TEMPERATURE_CALLBACK_PERIOD = 2;
		private static byte FUNCTION_GET_TEMPERATURE_CALLBACK_PERIOD = 3;
		private static byte FUNCTION_SET_TEMPERATURE_CALLBACK_THRESHOLD = 4;
		private static byte FUNCTION_GET_TEMPERATURE_CALLBACK_THRESHOLD = 5;
		private static byte FUNCTION_SET_DEBOUNCE_PERIOD = 6;
		private static byte FUNCTION_GET_DEBOUNCE_PERIOD = 7;
		private static byte CALLBACK_TEMPERATURE = 8;
		private static byte CALLBACK_TEMPERATURE_REACHED = 9;


		/// <summary>
		///  This callback is triggered periodically with the period that is set by
		///  <see cref="Tinkerforge.BrickletTemperature.SetTemperatureCallbackPeriod"/>. The parameter is the temperature
		///  of the sensor.
		///  
		///  <see cref="Tinkerforge.BrickletTemperature.Temperature"/> is only triggered if the temperature has changed since the
		///  last triggering.
		/// </summary>
		public delegate void Temperature(short temperature);

		/// <summary>
		///  This callback is triggered when the threshold as set by
		///  <see cref="Tinkerforge.BrickletTemperature.SetTemperatureCallbackThreshold"/> is reached.
		///  The parameter is the temperature of the sensor.
		///  
		///  If the threshold keeps being reached, the callback is triggered periodically
		///  with the period as set by <see cref="Tinkerforge.BrickletTemperature.SetDebouncePeriod"/>.
		/// </summary>
		public delegate void TemperatureReached(short temperature);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickletTemperature(string uid) : base(uid) 
		{
			this.expectedName = "Temperature Bricklet";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 0;
			messageCallbacks[CALLBACK_TEMPERATURE] = new MessageCallback(CallbackTemperature);
			messageCallbacks[CALLBACK_TEMPERATURE_REACHED] = new MessageCallback(CallbackTemperatureReached);
		}

		/// <summary>
		///  Returns the temperature of the sensor. The value
		///  has a range of -2500 to 8500 and is given in °C/100,
		///  e.g. a value of 4223 means that a temperature of 42.23 °C is measured.
		///  
		///  If you want to get the temperature periodically, it is recommended 
		///  to use the callback <see cref="Tinkerforge.BrickletTemperature.Temperature"/> and set the period with 
		///  <see cref="Tinkerforge.BrickletTemperature.SetTemperatureCallbackPeriod"/>.
		/// </summary>
		public short GetTemperature()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_TEMPERATURE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_TEMPERATURE, out answer);

			return LEConverter.ShortFrom(4, answer);
		}

		/// <summary>
		///  Sets the period in ms with which the <see cref="Tinkerforge.BrickletTemperature.Temperature"/> callback is triggered
		///  periodically. A value of 0 turns the callback off.
		///  
		///  <see cref="Tinkerforge.BrickletTemperature.Temperature"/> is only triggered if the temperature has changed since the
		///  last triggering.
		///  
		///  The default value is 0.
		/// </summary>
		public void SetTemperatureCallbackPeriod(uint period)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_TEMPERATURE_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(period, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the period as set by <see cref="Tinkerforge.BrickletTemperature.SetTemperatureCallbackPeriod"/>.
		/// </summary>
		public uint GetTemperatureCallbackPeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_TEMPERATURE_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_TEMPERATURE_CALLBACK_PERIOD, out answer);

			return LEConverter.UIntFrom(4, answer);
		}

		/// <summary>
		///  Sets the thresholds for the <see cref="Tinkerforge.BrickletTemperature.TemperatureReached"/> callback. 
		///  
		///  The following options are possible:
		///  
		///  <code>
		///   :header: "Option", "Description"
		///   :widths: 10, 100
		///   "'x'", "Callback is turned off."
		///   "'o'", "Callback is triggered when the temperature is *outside* the min and max values"
		///   "'i'", "Callback is triggered when the temperature is *inside* the min and max values"
		///   "'&lt;'", "Callback is triggered when the temperature is smaller than the min value (max is ignored)"
		///   "'&gt;'", "Callback is triggered when the temperature is greater than the min value (max is ignored)"
		///  </code>
		///  
		///  The default value is ('x', 0, 0).
		/// </summary>
		public void SetTemperatureCallbackThreshold(char option, short min, short max)
		{
			byte[] data_ = new byte[9];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_TEMPERATURE_CALLBACK_THRESHOLD, 1, data_);
			LEConverter.To((ushort)9, 2, data_);
			LEConverter.To(option, 4, data_);
			LEConverter.To(min, 5, data_);
			LEConverter.To(max, 7, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the threshold as set by <see cref="Tinkerforge.BrickletTemperature.SetTemperatureCallbackThreshold"/>.
		/// </summary>
		public void GetTemperatureCallbackThreshold(out char option, out short min, out short max)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_TEMPERATURE_CALLBACK_THRESHOLD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_TEMPERATURE_CALLBACK_THRESHOLD, out answer);

			option = LEConverter.CharFrom(4, answer);
			min = LEConverter.ShortFrom(5, answer);
			max = LEConverter.ShortFrom(7, answer);
		}

		/// <summary>
		///  Sets the period in ms with which the threshold callback
		///  
		///   <see cref="Tinkerforge.BrickletTemperature.TemperatureReached"/>
		///  
		///  is triggered, if the threshold
		///  
		///   <see cref="Tinkerforge.BrickletTemperature.SetTemperatureCallbackThreshold"/>
		///  
		///  keeps being reached.
		///  
		///  The default value is 100.
		/// </summary>
		public void SetDebouncePeriod(uint debounce)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_DEBOUNCE_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(debounce, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the debounce period as set by <see cref="Tinkerforge.BrickletTemperature.SetDebouncePeriod"/>.
		/// </summary>
		public uint GetDebouncePeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_DEBOUNCE_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_DEBOUNCE_PERIOD, out answer);

			return LEConverter.UIntFrom(4, answer);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetTemperature(out short temperature)
		{
			temperature = GetTemperature();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetTemperatureCallbackPeriod(out uint period)
		{
			period = GetTemperatureCallbackPeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetDebouncePeriod(out uint debounce)
		{
			debounce = GetDebouncePeriod();
		}

		internal int CallbackTemperature(byte[] data_)
		{
			short temperature = LEConverter.ShortFrom(4, data_);

			((Temperature)callbacks[CALLBACK_TEMPERATURE])(temperature);
			return 6;
		}

		internal int CallbackTemperatureReached(byte[] data_)
		{
			short temperature = LEConverter.ShortFrom(4, data_);

			((TemperatureReached)callbacks[CALLBACK_TEMPERATURE_REACHED])(temperature);
			return 6;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(Temperature))
			{
				callbacks[CALLBACK_TEMPERATURE] = d;
			}
			else if(d.GetType() == typeof(TemperatureReached))
			{
				callbacks[CALLBACK_TEMPERATURE_REACHED] = d;
			}
		}
	}
}
