/* ***********************************************************
 * This file was automatically generated on 2012-05-18.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for non-contact temperature sensing
	/// </summary>
	public class BrickletTemperatureIR : Device 
	{
		private static byte FUNCTION_GET_AMBIENT_TEMPERATURE = 1;
		private static byte FUNCTION_GET_OBJECT_TEMPERATURE = 2;
		private static byte FUNCTION_SET_EMISSIVITY = 3;
		private static byte FUNCTION_GET_EMISSIVITY = 4;
		private static byte FUNCTION_SET_AMBIENT_TEMPERATURE_CALLBACK_PERIOD = 5;
		private static byte FUNCTION_GET_AMBIENT_TEMPERATURE_CALLBACK_PERIOD = 6;
		private static byte FUNCTION_SET_OBJECT_TEMPERATURE_CALLBACK_PERIOD = 7;
		private static byte FUNCTION_GET_OBJECT_TEMPERATURE_CALLBACK_PERIOD = 8;
		private static byte FUNCTION_SET_AMBIENT_TEMPERATURE_CALLBACK_THRESHOLD = 9;
		private static byte FUNCTION_GET_AMBIENT_TEMPERATURE_CALLBACK_THRESHOLD = 10;
		private static byte FUNCTION_SET_OBJECT_TEMPERATURE_CALLBACK_THRESHOLD = 11;
		private static byte FUNCTION_GET_OBJECT_TEMPERATURE_CALLBACK_THRESHOLD = 12;
		private static byte FUNCTION_SET_DEBOUNCE_PERIOD = 13;
		private static byte FUNCTION_GET_DEBOUNCE_PERIOD = 14;
		private static byte CALLBACK_AMBIENT_TEMPERATURE = 15;
		private static byte CALLBACK_OBJECT_TEMPERATURE = 16;
		private static byte CALLBACK_AMBIENT_TEMPERATURE_REACHED = 17;
		private static byte CALLBACK_OBJECT_TEMPERATURE_REACHED = 18;


		/// <summary>
		///  This callback is triggered periodically with the period that is set by
		///  <see cref="Tinkerforge.BrickletTemperatureIR.SetAmbientTemperatureCallbackPeriod"/>. The parameter is the ambient
		///  temperature of the sensor.
		///  
		///  <see cref="Tinkerforge.BrickletTemperatureIR.AmbientTemperature"/> is only triggered if the ambient temperature
		///  has changed since the last triggering.
		/// </summary>
		public delegate void AmbientTemperature(short temperature);

		/// <summary>
		///  This callback is triggered periodically with the period that is set by
		///  <see cref="Tinkerforge.BrickletTemperatureIR.SetObjectTemperatureCallbackPeriod"/>. The parameter is the object
		///  temperature of the sensor.
		///  
		///  <see cref="Tinkerforge.BrickletTemperatureIR.AmbientTemperature"/> is only triggered if the object temperature
		///  has changed since the last triggering.
		/// </summary>
		public delegate void ObjectTemperature(short temperature);

		/// <summary>
		///  This callback is triggered when the threshold as set by
		///  <see cref="Tinkerforge.BrickletTemperatureIR.SetAmbientTemperatureCallbackThreshold"/> is reached.
		///  The parameter is the ambient temperature of the sensor.
		///  
		///  If the threshold keeps being reached, the callback is triggered periodically
		///  with the period as set by <see cref="Tinkerforge.BrickletTemperatureIR.SetDebouncePeriod"/>.
		/// </summary>
		public delegate void AmbientTemperatureReached(short temperature);

		/// <summary>
		///  This callback is triggered when the threshold as set by
		///  <see cref="Tinkerforge.BrickletTemperatureIR.SetObjectTemperatureCallbackThreshold"/> is reached.
		///  The parameter is the object temperature of the sensor.
		///  
		///  If the threshold keeps being reached, the callback is triggered periodically
		///  with the period as set by <see cref="Tinkerforge.BrickletTemperatureIR.SetDebouncePeriod"/>.
		/// </summary>
		public delegate void ObjectTemperatureReached(short temperature);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickletTemperatureIR(string uid) : base(uid) 
		{
			this.expectedName = "Temperature IR Bricklet";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 0;
			messageCallbacks[CALLBACK_AMBIENT_TEMPERATURE] = new MessageCallback(CallbackAmbientTemperature);
			messageCallbacks[CALLBACK_OBJECT_TEMPERATURE] = new MessageCallback(CallbackObjectTemperature);
			messageCallbacks[CALLBACK_AMBIENT_TEMPERATURE_REACHED] = new MessageCallback(CallbackAmbientTemperatureReached);
			messageCallbacks[CALLBACK_OBJECT_TEMPERATURE_REACHED] = new MessageCallback(CallbackObjectTemperatureReached);
		}

		/// <summary>
		///  Returns the ambient temperature of the sensor. The value
		///  has a range of -400 to 1250 and is given in °C/10,
		///  e.g. a value of 423 means that an ambient temperature of 42.3 °C is 
		///  measured.
		///  
		///  If you want to get the ambient temperature periodically, it is recommended 
		///  to use the callback <see cref="Tinkerforge.BrickletTemperatureIR.AmbientTemperature"/> and set the period with 
		///  <see cref="Tinkerforge.BrickletTemperatureIR.SetAmbientTemperatureCallbackPeriod"/>.
		/// </summary>
		public short GetAmbientTemperature()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_AMBIENT_TEMPERATURE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_AMBIENT_TEMPERATURE, out answer);

			return LEConverter.ShortFrom(4, answer);
		}

		/// <summary>
		///  Returns the object temperature of the sensor, i.e. the temperature
		///  of the surface of the object the sensor is aimed at. The value
		///  has a range of -700 to 3800 and is given in °C/10,
		///  e.g. a value of 30001 means that a temperature of 300.01 °C is measured
		///  on the surface of the object.
		///  
		///  The temperature of different materials is dependent on their `emissivity 
		///  &lt;http://en.wikipedia.org/wiki/Emissivity&gt;`_. The emissivity of the material
		///  can be set with <see cref="Tinkerforge.BrickletTemperatureIR.SetEmissivity"/>.
		///  
		///  If you want to get the object temperature periodically, it is recommended 
		///  to use the callback <see cref="Tinkerforge.BrickletTemperatureIR.ObjectTemperature"/> and set the period with 
		///  <see cref="Tinkerforge.BrickletTemperatureIR.SetObjectTemperatureCallbackPeriod"/>.
		/// </summary>
		public short GetObjectTemperature()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_OBJECT_TEMPERATURE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_OBJECT_TEMPERATURE, out answer);

			return LEConverter.ShortFrom(4, answer);
		}

		/// <summary>
		///  Sets the `emissivity &lt;http://en.wikipedia.org/wiki/Emissivity&gt;`_ that is 
		///  used to calculate the surface temperature as returned by 
		///  <see cref="Tinkerforge.BrickletTemperatureIR.GetObjectTemperature"/>. 
		///  
		///  The emissivity is usually given as a value between 0.0 and 1.0. A list of
		///  emissivities of different materials can be found 
		///  `here &lt;http://www.infrared-thermography.com/material.htm&gt;`_.
		///  
		///  The parameter of <see cref="Tinkerforge.BrickletTemperatureIR.SetEmissivity"/> has to be given with a factor of
		///  65535 (16 bit). For example: An emissivity of 0.1 can be set with the 
		///  value 6553, an emissivity of 0.5 with the value 32767 and so on.
		///  
		///  <note>
		///    If you need a precise measurement for the object temperature, it is
		///    absolutely crucial that you also provide a precise emissivity.
		///  </note>
		///  
		///  The default emissivity is 1.0 (value of 65535) and the minimum emissivity the
		///  sensor can handle is 0.1 (value of 6553).
		/// </summary>
		public void SetEmissivity(ushort emissivity)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_EMISSIVITY, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(emissivity, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the emissivity as set by <see cref="Tinkerforge.BrickletTemperatureIR.SetEmissivity"/>.
		/// </summary>
		public ushort GetEmissivity()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_EMISSIVITY, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_EMISSIVITY, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Sets the period in ms with which the <see cref="Tinkerforge.BrickletTemperatureIR.AmbientTemperature"/> callback is triggered
		///  periodically. A value of 0 turns the callback off.
		///  
		///  <see cref="Tinkerforge.BrickletTemperatureIR.AmbientTemperature"/> is only triggered if the temperature has changed since the
		///  last triggering.
		///  
		///  The default value is 0.
		/// </summary>
		public void SetAmbientTemperatureCallbackPeriod(uint period)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_AMBIENT_TEMPERATURE_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(period, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the period as set by <see cref="Tinkerforge.BrickletTemperatureIR.SetAmbientTemperatureCallbackPeriod"/>.
		/// </summary>
		public uint GetAmbientTemperatureCallbackPeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_AMBIENT_TEMPERATURE_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_AMBIENT_TEMPERATURE_CALLBACK_PERIOD, out answer);

			return LEConverter.UIntFrom(4, answer);
		}

		/// <summary>
		///  Sets the period in ms with which the <see cref="Tinkerforge.BrickletTemperatureIR.ObjectTemperature"/> callback is triggered
		///  periodically. A value of 0 turns the callback off.
		///  
		///  <see cref="Tinkerforge.BrickletTemperatureIR.ObjectTemperature"/> is only triggered if the temperature has changed since the
		///  last triggering.
		///  
		///  The default value is 0.
		/// </summary>
		public void SetObjectTemperatureCallbackPeriod(uint period)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_OBJECT_TEMPERATURE_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(period, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the period as set by <see cref="Tinkerforge.BrickletTemperatureIR.SetObjectTemperatureCallbackPeriod"/>.
		/// </summary>
		public uint GetObjectTemperatureCallbackPeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_OBJECT_TEMPERATURE_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_OBJECT_TEMPERATURE_CALLBACK_PERIOD, out answer);

			return LEConverter.UIntFrom(4, answer);
		}

		/// <summary>
		///  Sets the thresholds for the <see cref="Tinkerforge.BrickletTemperatureIR.AmbientTemperatureReached"/> callback. 
		///  
		///  The following options are possible:
		///  
		///  <code>
		///   :header: "Option", "Description"
		///   :widths: 10, 100
		///   "'x'", "Callback is turned off."
		///   "'o'", "Callback is triggered when the temperature is *outside* the min and max values"
		///   "'i'", "Callback is triggered when the temperature is *inside* the min and max values"
		///   "'&lt;'", "Callback is triggered when the temperature is smaller than the min value (max is ignored)"
		///   "'&gt;'", "Callback is triggered when the temperature is greater than the min value (max is ignored)"
		///  </code>
		///  
		///  The default value is ('x', 0, 0).
		/// </summary>
		public void SetAmbientTemperatureCallbackThreshold(char option, short min, short max)
		{
			byte[] data_ = new byte[9];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_AMBIENT_TEMPERATURE_CALLBACK_THRESHOLD, 1, data_);
			LEConverter.To((ushort)9, 2, data_);
			LEConverter.To(option, 4, data_);
			LEConverter.To(min, 5, data_);
			LEConverter.To(max, 7, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the threshold as set by <see cref="Tinkerforge.BrickletTemperatureIR.SetAmbientTemperatureCallbackThreshold"/>.
		/// </summary>
		public void GetAmbientTemperatureCallbackThreshold(out char option, out short min, out short max)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_AMBIENT_TEMPERATURE_CALLBACK_THRESHOLD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_AMBIENT_TEMPERATURE_CALLBACK_THRESHOLD, out answer);

			option = LEConverter.CharFrom(4, answer);
			min = LEConverter.ShortFrom(5, answer);
			max = LEConverter.ShortFrom(7, answer);
		}

		/// <summary>
		///  Sets the thresholds for the <see cref="Tinkerforge.BrickletTemperatureIR.ObjectTemperatureReached"/> callback. 
		///  
		///  The following options are possible:
		///  
		///  <code>
		///   :header: "Option", "Description"
		///   :widths: 10, 100
		///   "'x'", "Callback is turned off."
		///   "'o'", "Callback is triggered when the temperature is *outside* the min and max values"
		///   "'i'", "Callback is triggered when the temperature is *inside* the min and max values"
		///   "'&lt;'", "Callback is triggered when the temperature is smaller than the min value (max is ignored)"
		///   "'&gt;'", "Callback is triggered when the temperature is greater than the min value (max is ignored)"
		///  </code>
		///  
		///  The default value is ('x', 0, 0).
		/// </summary>
		public void SetObjectTemperatureCallbackThreshold(char option, short min, short max)
		{
			byte[] data_ = new byte[9];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_OBJECT_TEMPERATURE_CALLBACK_THRESHOLD, 1, data_);
			LEConverter.To((ushort)9, 2, data_);
			LEConverter.To(option, 4, data_);
			LEConverter.To(min, 5, data_);
			LEConverter.To(max, 7, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the threshold as set by <see cref="Tinkerforge.BrickletTemperatureIR.SetAmbientTemperatureCallbackThreshold"/>.
		/// </summary>
		public void GetObjectTemperatureCallbackThreshold(out char option, out short min, out short max)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_OBJECT_TEMPERATURE_CALLBACK_THRESHOLD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_OBJECT_TEMPERATURE_CALLBACK_THRESHOLD, out answer);

			option = LEConverter.CharFrom(4, answer);
			min = LEConverter.ShortFrom(5, answer);
			max = LEConverter.ShortFrom(7, answer);
		}

		/// <summary>
		///  Sets the period in ms with which the threshold callbacks
		///  
		///   <see cref="Tinkerforge.BrickletTemperatureIR.AmbientTemperatureReached"/>, <see cref="Tinkerforge.BrickletTemperatureIR.ObjectTemperatureReached"/>
		///  
		///  are triggered, if the thresholds
		///  
		///   <see cref="Tinkerforge.BrickletTemperatureIR.SetAmbientTemperatureCallbackThreshold"/>, <see cref="Tinkerforge.BrickletTemperatureIR.SetObjectTemperatureCallbackThreshold"/>
		///  
		///  keep being reached.
		///  
		///  The default value is 100.
		/// </summary>
		public void SetDebouncePeriod(uint debounce)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_DEBOUNCE_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(debounce, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the debounce period as set by <see cref="Tinkerforge.BrickletTemperatureIR.SetDebouncePeriod"/>.
		/// </summary>
		public uint GetDebouncePeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_DEBOUNCE_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_DEBOUNCE_PERIOD, out answer);

			return LEConverter.UIntFrom(4, answer);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetAmbientTemperature(out short temperature)
		{
			temperature = GetAmbientTemperature();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetObjectTemperature(out short temperature)
		{
			temperature = GetObjectTemperature();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetEmissivity(out ushort emissivity)
		{
			emissivity = GetEmissivity();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetAmbientTemperatureCallbackPeriod(out uint period)
		{
			period = GetAmbientTemperatureCallbackPeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetObjectTemperatureCallbackPeriod(out uint period)
		{
			period = GetObjectTemperatureCallbackPeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetDebouncePeriod(out uint debounce)
		{
			debounce = GetDebouncePeriod();
		}

		internal int CallbackAmbientTemperature(byte[] data_)
		{
			short temperature = LEConverter.ShortFrom(4, data_);

			((AmbientTemperature)callbacks[CALLBACK_AMBIENT_TEMPERATURE])(temperature);
			return 6;
		}

		internal int CallbackObjectTemperature(byte[] data_)
		{
			short temperature = LEConverter.ShortFrom(4, data_);

			((ObjectTemperature)callbacks[CALLBACK_OBJECT_TEMPERATURE])(temperature);
			return 6;
		}

		internal int CallbackAmbientTemperatureReached(byte[] data_)
		{
			short temperature = LEConverter.ShortFrom(4, data_);

			((AmbientTemperatureReached)callbacks[CALLBACK_AMBIENT_TEMPERATURE_REACHED])(temperature);
			return 6;
		}

		internal int CallbackObjectTemperatureReached(byte[] data_)
		{
			short temperature = LEConverter.ShortFrom(4, data_);

			((ObjectTemperatureReached)callbacks[CALLBACK_OBJECT_TEMPERATURE_REACHED])(temperature);
			return 6;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(AmbientTemperature))
			{
				callbacks[CALLBACK_AMBIENT_TEMPERATURE] = d;
			}
			else if(d.GetType() == typeof(ObjectTemperature))
			{
				callbacks[CALLBACK_OBJECT_TEMPERATURE] = d;
			}
			else if(d.GetType() == typeof(AmbientTemperatureReached))
			{
				callbacks[CALLBACK_AMBIENT_TEMPERATURE_REACHED] = d;
			}
			else if(d.GetType() == typeof(ObjectTemperatureReached))
			{
				callbacks[CALLBACK_OBJECT_TEMPERATURE_REACHED] = d;
			}
		}
	}
}
